/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 16/05/2007
   derniere modification : 05/03/2008 */

#include <SCE/SCEMinimal.h>

#include <SCE/core/SCECore.h>
#include <SCE/interface/SCEModel.h>

/* modele binde pour le rendu */
static SCE_SModel *binded = NULL;


int SCE_Init_Model (void)
{
    return SCE_OK;
}
void SCE_Quit_Model (void)
{
}


/* revise le 09/03/2008 */
void SCE_Model_Init (SCE_SModel *mdl)
{
    mdl->meshs = NULL;
    mdl->n_meshs = 0;
    mdl->lod = 0;
    mdl->use_lod = SCE_FALSE;
    mdl->lodfunc = NULL;
    mdl->lodfuncparam = NULL;

    mdl->mat = NULL;
    mdl->cullface = SCE_TRUE; /* backface culling active par defaut */
    mdl->node = NULL;
    SCE_BoundingBox_Init (&mdl->box);
}


/* revise le 02/03/2008 */
SCE_SModel* SCE_Model_Create (void)
{
    SCE_SModel *mdl = NULL;

    SCE_btstart ();
    mdl = SCE_malloc (sizeof *mdl);
    if (!mdl)
        Logger_LogSrc ();
    else
    {
        SCE_Model_Init (mdl);
        if (!(mdl->mat = SCE_Material_Create ()))
        {
            SCE_Model_Delete (mdl);
            Logger_LogSrc ();
            SCE_btend ();
            return NULL;
        }
        if (!(mdl->node = SCE_Node_Create ()))
        {
            SCE_Model_Delete (mdl), mdl = NULL;
            Logger_LogSrc ();
        }
    }
    SCE_btend ();
    return mdl;
}

/* revise le 02/03/2008 */
void SCE_Model_Delete (SCE_SModel *mdl)
{
    SCE_btstart ();
    if (mdl)
    {
        SCE_Mesh_DeleteList (mdl->meshs);
        SCE_Material_Delete (mdl->mat);
        SCE_Node_Delete (mdl->node);
        SCE_free (mdl);
    }
    SCE_btend ();
}


/* ajoute le 25/06/2007 */
/* revise le 02/03/2008 */
int SCE_Model_LoadMesh (SCE_SModel *mdl, const char *fname)
{
    SCE_SMesh **meshs = NULL;
    int n_meshs = 0;
    
    SCE_btstart ();
    meshs = SCE_Mesh_Load (fname, NULL);
    if (!meshs)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    SCE_Mesh_DeleteList (mdl->meshs);
    mdl->meshs = meshs;
    while (meshs[n_meshs])
        n_meshs++;
    mdl->n_meshs = n_meshs;
    SCE_btend ();
    return SCE_OK;
}


/* ajoute le 02/03/2008 */
void SCE_Model_ActiveMeshLOD (SCE_SModel *mdl, int use)
{
    if (mdl->lodfunc && use)
        mdl->use_lod = use;
    else
        mdl->use_lod = SCE_FALSE;
}
void SCE_Model_EnableMeshLOD (SCE_SModel *mdl)
{
    if (mdl->lodfunc)
        mdl->use_lod = SCE_TRUE;
}
void SCE_Model_DisableMeshLOD (SCE_SModel *mdl)
{
    mdl->use_lod = SCE_FALSE;
}
/* ajoute le 02/03/2008 */
void SCE_Model_SetMeshLODFunc (SCE_SModel *mdl, SCE_FMeshLODFunc func, void *p)
{
    mdl->lodfunc = func;
    if (!func)
        mdl->use_lod = SCE_FALSE;
    mdl->lodfuncparam = p;
}
/* ajoute le 02/03/2008 */
void SCE_Model_UpdateMeshLOD (SCE_SModel *mdl)
{
    if (mdl->use_lod)
    {
        mdl->lod = mdl->lodfunc (mdl, mdl->lodfuncparam);
        if (mdl->lod >= mdl->n_meshs)
            mdl->lod = mdl->n_meshs - 1;
    }
    else
        mdl->lod = 0;
}

/* ajoute le 05/03/2008 */
unsigned int SCE_Model_GetMeshLOD (SCE_SModel *mdl)
{
    return mdl->lod;
}
/* ajoute le 02/03/2008 */
SCE_SNode* SCE_Model_GetNode (SCE_SModel *mdl)
{
    return mdl->node;
}
/* ajoute le 02/03/2008 */
SCE_SBoundingBox* SCE_Model_GetBoundingBox (SCE_SModel *mdl)
{
    return &mdl->box;
}
/* ajoute le 02/03/2008 */
SCE_SMesh** SCE_Model_GetMeshs (SCE_SModel *mdl)
{
    return mdl->meshs;
}
/* ajoute le 02/03/2008 */
SCE_SMesh* SCE_Model_GetMesh (SCE_SModel *mdl)
{
    SCE_Model_UpdateMeshLOD (mdl);
    return mdl->meshs[mdl->lod];
}
/* ajoute le 02/03/2008 */
SCE_SMaterial* SCE_Model_GetMaterial (SCE_SModel *mdl)
{
    return mdl->mat;
}

/* ajoute le 02/03/2008 */
int SCE_Model_GenerateBoundingBox (SCE_SModel *mdl)
{
    SCE_SBoundingBox box;

    SCE_btstart ();
    SCE_BoundingBox_Init (&box);
    if (SCE_Mesh_GenerateBoundingBox (mdl->meshs[0/*..?*/], &box) < 0)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    mdl->box = box;
    SCE_btend ();
    return SCE_OK;
}

/* ajoute le 14/06/2007 */
/* revise le 15/01/2008 */
void SCE_Model_Bind (SCE_SModel *mdl)
{
    binded = mdl;
    if (mdl)
    {
        SCE_Mesh_Use (mdl->meshs[mdl->lod]);
        SCE_Material_Use (mdl->mat);
    }
    else
    {
        SCE_Material_Use (NULL);
        SCE_Mesh_Use (NULL);
    }
}

/* ajoute le 14/06/2007 */
/* revise le 15/01/2008 */
void SCE_Model_BindMesh (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Mesh_Use (mdl->meshs[mdl->lod]);
    else
        SCE_Mesh_Use (NULL);
}

/* ajoute le 14/06/2007 */
void SCE_Model_BindMaterial (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Material_Use (mdl->mat);
    else
        SCE_Material_Use (NULL);
}

/* ajoute le 04/03/2008 */
void SCE_Model_BindNode (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Node_Use (mdl->node);
    else
        SCE_Node_Use (NULL);
}

/* ajoute le 14/06/2007 */
void SCE_Model_Draw (void)
{
    SCE_Mesh_Draw ();
}

/* revise le 02/03/2008 */
void SCE_Model_Render (SCE_SModel *mdl)
{
    SCE_CSetState (GL_CULL_FACE, mdl->cullface);
    SCE_Model_Bind (mdl);
    SCE_CPushMatrix ();
    SCE_Node_Use (mdl->node);
    SCE_Model_Draw ();
    SCE_Node_Use (NULL);
    SCE_CPopMatrix ();
    SCE_Model_Bind (NULL);
}
